# -*- coding: utf-8 -*-

import urllib
import re
import logging
import urllib
from inspect import isclass
from webob.multidict import MultiDict

logger = logging.getLogger(__name__)


def urlquote(value):
    return urllib.quote(value.encode('utf-8') if isinstance(value, unicode) else str(value))


def construct_url(path, query, host, port, schema):
    query = ('?' + '&'.join(['%s=%s' % (urlquote(k), urlquote(v)) \
                            for k,v in query.iteritems()])  \
             if query else '')

    path = path
    if host:
        host = host.encode('idna')
        port = ':' + port if port else ''
        return ''.join((schema, '://', host, port, path,  query))
    else:
        return path + query


class URL(str):

    def __new__(cls, path, query=None, host=None, port=None, schema=None):
        '''
        path - urlencoded string or unicode object (not encoded at all)
        '''
        path = path if isinstance(path, str) else urlquote(path)
        query = MultiDict(query) if query else MultiDict()
        host = host or ''
        port = port or ''
        schema = schema or 'http'
        self = str.__new__(cls, construct_url(path, query, host, 
                                              port,schema))
        self.path = path
        self.query = query
        self.host = host
        self.port = port
        self.schema = schema
        return self

    def _copy(self, **kwargs):
        path = kwargs.pop('path', self.path)
        kw = dict(query=self.query, host=self.host, 
                  port=self.port, schema=self.schema)
        kw.update(kwargs)
        return self.__class__(path, **kw)

    def set(self, **kwargs):
        '''Sets value of URL's query keys to given values'''
        query = self.query.copy()
        for k, v in kwargs.items():
            query[k] = v
        return self._copy(query=query)

    def add(self, **kwargs):
        '''Adds values to URL's query'''
        query = self.query.copy()
        for k, v in kwargs.items():
            query.add(k, v)
        return self._copy(query=query)

    def delete(self, key):
        '''Deletes given key from the URL's query'''
        query = self.query.copy()
        del query[key]
        return self._copy(query=query)

    def getall(self, key):
        '''A proxy method for query.getall'''
        return self.query.getall(key)

    def getone(self, key):
        '''A proxy method for query.getone'''
        return self.query.getone(key)

    def get(self, key, default=None):
        '''A proxy method for query.get'''
        return self.query.get(key, default=default)

    def get_readable(self):
        '''Gets human-readable representation of the url'''
        query = (u'?' + u'&'.join([u'%s=%s' % (k,v) for k, v in self.query.iteritems()]) \
                 if self.query else '')

        path = urllib.unquote(self.path).decode('utf-8')
        if self.host:
            port = u':' + self.port if self.port else u''
            return u''.join((self.schema, '://', self.host, port, path,  query))
        else:
            return path + query

    def __repr__(self):
        return '<URL %r>' % str.__repr__(self)


class ConvertError(Exception):

    @property
    def converter(self):
        return self.args[0]

    @property
    def value(self):
        return self.args[1]


class Converter(object):
    '''A base class for urlconverters'''

    #: A key significating what converter is used in particular url template
    name=None

    def to_python(self, value, **kwargs):
        '''
        Accepts unicode url part and returns python object.
        Should be implemented in subclasses
        '''
        raise NotImplementedError()

    def to_url(self, value):
        '''
        Accepts python object and returns unicode prepared to be used
        in url building.
        Should be implemented in subclasses
        '''
        raise NotImplementedError()


class String(Converter):
    '''
    Unquotes urlencoded string.

    The converter's name is 'string'
    '''

    name='string'

    def to_python(self, value, **kwargs):
        return value

    def to_url(self, value):
        return unicode(value)


class Integer(Converter):
    '''
    Extracts integer value from url part.

    The converter's name is 'int'
    '''

    name='int'

    def to_python(self, value, **kwargs):
        try:
            value = int(value)
        except ValueError:
            raise ConvertError(self.name, value)
        else:
            return value

    def to_url(self, value):
        return str(value)


class Boolean(Converter):
    '''
    Translates on/off, true/false, True/False, yes/no strings to python bool.

    The converter's name is 'bool'.
    '''

    name='bool'
    _true = ['on', 'true', 'True', 'yes']
    _false = ['off', 'false', 'False', 'no']

    def to_python(self, value, **kwargs):
        if value in self._true:
            return True
        elif value in self._false:
            return False
        raise ConvertError(self.name, value)

    def to_url(self, value):
        if value:
            return 'yes'
        return 'no'


class Any(Converter):
    name='any'
    def __init__(self, *values):
        self.values = values

    def to_python(self, value, **kwargs):
        if value in self.values:
            return value
        raise ConvertError(self.name, value)


convs_dict = dict((item.name or item.__name__, item) \
                  for item in globals().values() \
                  if isclass(item) and issubclass(item, Converter))


_split_pattern = re.compile(r'(<[^<]*>)')

#NOTE: taken from werkzeug
_converter_pattern = re.compile(r'''^<
        (?P<converter>[a-zA-Z_][a-zA-Z0-9]+)    # converter name
        (?:\((?P<args>.*?)\))?                  # converter args
        \:?                                     # delimiter
        (?P<variable>[a-zA-Z_][a-zA-Z0-9_]*)?   # variable name
        >$''', re.VERBOSE | re.U)

_static_url_pattern = re.compile(r'^[^<]*?$')

def construct_re(url_template, match_whole_str=False, converters=None,
                 default_converter='string'):
    '''
    url_template - str or unicode representing template

    Constructed pattern expects urlencoded string!

    returns  (compiled re pattern, 
              dict {url param name: [converter name, converter args (str)]},
              list of (variable name, converter name, converter args name))
    '''
    converters = converters or {}
    converters.update(convs_dict)
    # needed for reverse url building (or not needed?)
    builder_params = []
    # found url params and their converters
    url_params = {}
    result = r'^'
    parts = _split_pattern.split(url_template)
    total_parts = len(parts)
    for i, part in enumerate(parts):
        if part:
            is_url_pattern = _static_url_pattern.match(part)
            if is_url_pattern:
                #NOTE: right order:
                #      - make part str if it was unicode
                #      - urlquote part
                #      - escape all specific for re chars in part
                part = urlquote(unicode(part).encode('utf-8'))
                result += re.escape(part)
                builder_params.append(part)
                continue
            is_converter = _converter_pattern.match(part)
            if is_converter:
                converter = is_converter.groupdict()['converter']
                args = is_converter.groupdict()['args']
                variable = is_converter.groupdict()['variable']
                if variable is None:
                    variable = converter
                    converter = default_converter
                result += '(?P<%s>[.a-zA-Z0-9_%%-]+)' % variable
                try:
                    conv_object = init_converter(converters[converter], args)
                except KeyError:
                    raise KeyError('There is no converter named "%s"' % converter)
                builder_params.append((variable, conv_object))
                url_params[variable] = conv_object
                continue
            raise ValueError('Incorrect url template "%s"' % url_template)
        else:
            if i < total_parts - 1:
                raise ValueError('Incorrect url template "%s"' % url_template)
    if match_whole_str:
        result += '$'
    return re.compile(result), url_params, builder_params


def init_converter(conv_class, args):
    if args:
        #XXX: taken from werkzeug
        storage = type('_Storage', (), {'__getitem__': lambda s, x: x})()
        args, kwargs = eval(u'(lambda *a, **kw: (a, kw))(%s)' % args, {}, storage)
        return conv_class(*args, **kwargs)
    return conv_class()


class UrlTemplate(object):

    def __init__(self, template, match_whole_str=True, converters=None,
                 default_converter='string'):
        self.template = template
        self.match_whole_str = match_whole_str
        self._allowed_converters = self._init_converters(converters)
        self._pattern, self._url_params, self._builder_params = construct_re(template, 
                                                                             match_whole_str=match_whole_str,
                                                                             converters=self._allowed_converters,
                                                                             default_converter=default_converter)

    def match(self, path, **kw):
        '''
        path - str (urlencoded)
        '''
        m = self._pattern.match(path)
        if m:
            kwargs = m.groupdict()
            # convert params
            for url_arg_name, value_urlencoded in kwargs.items():
                conv_obj = self._url_params[url_arg_name]
                unicode_value = urllib.unquote(value_urlencoded).decode('utf-8', 'replace')
                try:
                    kwargs[url_arg_name] = conv_obj.to_python(unicode_value, **kw)
                except ConvertError, err:
                    logger.debug('ConvertError by "%s", value "%s"' % (err.converter, err.value.encode('utf-8')))
                    return False, {}
            return True, kwargs
        return False, {}

    def __call__(self, **kwargs):
        'Url building with url params values taken from kwargs. (reverse)'
        result = ''
        for part in self._builder_params:
            if isinstance(part, tuple):
                var, conv_obj = part
                value = kwargs[var]
                result += urlquote(conv_obj.to_url(value))
            else:
                result += part
        # result - urlencoded str
        return result

    def _init_converters(self, converters):
        convs = convs_dict.copy()
        if converters is not None:
            for conv in converters:
                 name = conv.name or conv.__name__.lower()
                 convs[name] = conv
        return convs

    def __repr__(self):
        return '%s(%r, match_whole_str=%r)' % (self.__class__.__name__, 
                                               self.template.encode('utf-8'),
                                               self.match_whole_str)



def pare(text, size, etc='...'):
    '''Pare text to have maximum size and add etc to the end if it's
    changed'''
    size = int(size)
    text = text.strip()
    if len(text)>size:
        text = text[:size]
        half = size//2
        last = None
        import re
        whitespace = re.compile('\s+')
        for mo in whitespace.finditer(text[half:]):
            if mo is not None:
                last = mo
        if last is not None:
            text = text[:half+last.start()+1]
        return text+etc
    else:
        return text



# -*- coding: utf-8 -*-
"""
    OrderedDict
    ~~~~~~~~~~~

    This module is an example implementation of an ordered dict for the
    collections module.  It's not written for performance (it actually
    performs pretty bad) but to show how the API works.


    Questions and Answers
    =====================

    Why would anyone need ordered dicts?

        Dicts in python are unordered which means that the order of items when
        iterating over dicts is undefined.  As a matter of fact it is most of
        the time useless and differs from implementation to implementation.

        Many developers stumble upon that problem sooner or later when
        comparing the output of doctests which often does not match the order
        the developer thought it would.

        Also XML systems such as Genshi have their problems with unordered
        dicts as the input and output ordering of tag attributes is often
        mixed up because the ordering is lost when converting the data into
        a dict.  Switching to lists is often not possible because the
        complexity of a lookup is too high.

        Another very common case is metaprogramming.  The default namespace
        of a class in python is a dict.  With Python 3 it becomes possible
        to replace it with a different object which could be an ordered dict.
        Django is already doing something similar with a hack that assigns
        numbers to some descriptors initialized in the class body of a
        specific subclass to restore the ordering after class creation.

        When porting code from programming languages such as PHP and Ruby
        where the item-order in a dict is guaranteed it's also a great help
        to have an equivalent data structure in Python to ease the transition.

    Where are new keys added?

        At the end.  This behavior is consistent with Ruby 1.9 Hashmaps
        and PHP Arrays.  It also matches what common ordered dict
        implementations do currently.

    What happens if an existing key is reassigned?

        The key is *not* moved.  This is consitent with existing
        implementations and can be changed by a subclass very easily::

            class MovingOrderedDict(OrderedDict):
                def __setitem__(self, key, value):
                    self.pop(key, None)
                    OrderedDict.__setitem__(self, key, value)

        Moving keys to the end of a ordered dict on reassignment is not
        very useful for most applications.

    Does it mean the dict keys are sorted by a sort expression?

        That's not the case.  The OrderedDict only guarantees that there is an
        order and that newly inserted keys are inserted at the end of the dict.
        If you want to sort it you can do so, but newly added keys are again
        added at the end of the dict.

    I initializes the OrderedDict with a dict literal but the keys are not
    ordered like they should!

        Dict literals in Python generate dict objects and as such the order of
        their items is not guaranteed.  Before they are passed to the
        OrderedDict constructor they are already unordered.

    What happens if keys appear multiple times in the list passed to the
    constructor?

        The same as for the dict.  The latter item overrides the former.  This
        has the side-effect that the position of the first key is used because
        the key is actually overwritten:

        >>> OrderedDict([('a', 1), ('b', 2), ('a', 3)])
        OrderedDict([('a', 3), ('b', 2)])

        This behavor is consistent with existing implementation in Python
        and the PHP array and the hashmap in Ruby 1.9.

    This OrderedDict doesn't scale!

        Yes it doesn't.  The delitem operation is O(n).  This is file is a
        mockup of a real OrderedDict that could be implemented for collections
        based on an linked list.

    Why is there no .insert()?

        There are few situations where you really want to insert a key at
        an specified index.  To now make the API too complex the proposed
        solution for this situation is creating a list of items, manipulating
        that and converting it back into an OrderedDict:

        >>> d = OrderedDict([('a', 42), ('b', 23), ('c', 19)])
        >>> l = d.items()
        >>> l.insert(1, ('x', 0))
        >>> OrderedDict(l)
        OrderedDict([('a', 42), ('x', 0), ('b', 23), ('c', 19)])
"""
from itertools import izip, imap
from copy import deepcopy

missing = object()


class OrderedDict(dict):
    """
    Ordered dict example implementation.

    This is the proposed interface for a an ordered dict as proposed on the
    Python mailinglist (proposal_).

    It's a dict subclass and provides some list functions.  The implementation
    of this class is inspired by the implementation of Babel but incorporates
    some ideas from the `ordereddict`_ and Django's ordered dict.

    The constructor and `update()` both accept iterables of tuples as well as
    mappings:

    >>> d = OrderedDict([('a', 'b'), ('c', 'd')])
    >>> d.update({'foo': 'bar'})
    >>> d
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar')])
    
    Keep in mind that when updating from dict-literals the order is not
    preserved as these dicts are unsorted!

    You can copy an OrderedDict like a dict by using the constructor,
    `copy.copy` or the `copy` method and make deep copies with `copy.deepcopy`:

    >>> from copy import copy, deepcopy
    >>> copy(d)
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar')])
    >>> d.copy()
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar')])
    >>> OrderedDict(d)
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar')])
    >>> d['spam'] = []
    >>> d2 = deepcopy(d)
    >>> d2['spam'].append('eggs')
    >>> d
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar'), ('spam', [])])
    >>> d2
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar'), ('spam', ['eggs'])])

    All iteration methods as well as `keys`, `values` and `items` return
    the values ordered by the the time the key-value pair is inserted:

    >>> d.keys()
    ['a', 'c', 'foo', 'spam']
    >>> d.values()
    ['b', 'd', 'bar', []]
    >>> d.items()
    [('a', 'b'), ('c', 'd'), ('foo', 'bar'), ('spam', [])]
    >>> list(d.iterkeys())
    ['a', 'c', 'foo', 'spam']
    >>> list(d.itervalues())
    ['b', 'd', 'bar', []]
    >>> list(d.iteritems())
    [('a', 'b'), ('c', 'd'), ('foo', 'bar'), ('spam', [])]

    Index based lookup is supported too by `byindex` which returns the
    key/value pair for an index:

    >>> d.byindex(2)
    ('foo', 'bar')

    You can reverse the OrderedDict as well:

    >>> d.reverse()
    >>> d
    OrderedDict([('spam', []), ('foo', 'bar'), ('c', 'd'), ('a', 'b')])
    
    And sort it like a list:

    >>> d.sort(key=lambda x: x[0].lower())
    >>> d
    OrderedDict([('a', 'b'), ('c', 'd'), ('foo', 'bar'), ('spam', [])])

    .. _proposal: http://thread.gmane.org/gmane.comp.python.devel/95316
    .. _ordereddict: http://www.xs4all.nl/~anthon/Python/ordereddict/
    """

    def __init__(self, *args, **kwargs):
        dict.__init__(self)
        self._keys = []
        self.update(*args, **kwargs)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self._keys.remove(key)

    def __setitem__(self, key, item):
        if key not in self:
            self._keys.append(key)
        dict.__setitem__(self, key, item)

    def __deepcopy__(self, memo=None):
        if memo is None:
            memo = {}
        d = memo.get(id(self), missing)
        if d is not missing:
            return d
        memo[id(self)] = d = self.__class__()
        dict.__init__(d, deepcopy(self.items(), memo))
        d._keys = self._keys[:]
        return d

    def __getstate__(self):
        return {'items': dict(self), 'keys': self._keys}

    def __setstate__(self, d):
        self._keys = d['keys']
        dict.update(d['items'])

    def __reversed__(self):
        return reversed(self._keys)

    def __eq__(self, other):
        if isinstance(other, OrderedDict):
            if not dict.__eq__(self, other):
                return False
            return self.items() == other.items()
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __cmp__(self, other):
        if isinstance(other, OrderedDict):
            return cmp(self.items(), other.items())
        elif isinstance(other, dict):
            return dict.__cmp__(self, other)
        return NotImplemented

    @classmethod
    def fromkeys(cls, iterable, default=None):
        return cls((key, default) for key in iterable)

    def clear(self):
        del self._keys[:]
        dict.clear(self)

    def copy(self):
        return self.__class__(self)

    def items(self):
        return zip(self._keys, self.values())

    def iteritems(self):
        return izip(self._keys, self.itervalues())

    def keys(self):
        return self._keys[:]

    def iterkeys(self):
        return iter(self._keys)

    def pop(self, key, default=missing):
        if default is missing:
            return dict.pop(self, key)
        elif key not in self:
            return default
        self._keys.remove(key)
        return dict.pop(self, key, default)

    def popitem(self, key):
        self._keys.remove(key)
        return dict.popitem(key)

    def setdefault(self, key, default=None):
        if key not in self:
            self._keys.append(key)
        return dict.setdefault(self, key, default)

    def update(self, *args, **kwargs):
        sources = []
        if len(args) == 1:
            if hasattr(args[0], 'iteritems'):
                sources.append(args[0].iteritems())
            else:
                sources.append(iter(args[0]))
        elif args:
            raise TypeError('expected at most one positional argument')
        if kwargs:
            sources.append(kwargs.iteritems())
        for iterable in sources:
            for key, val in iterable:
                self[key] = val

    def values(self):
        return map(self.get, self._keys)

    def itervalues(self):
        return imap(self.get, self._keys)

    def index(self, item):
        return self._keys.index(item)

    def byindex(self, item):
        key = self._keys[item]
        return (key, dict.__getitem__(self, key))

    def reverse(self):
        self._keys.reverse()

    def sort(self, *args, **kwargs):
        self._keys.sort(*args, **kwargs)

    def __repr__(self):
        return 'OrderedDict(%r)' % self.items()

    __copy__ = copy
    __iter__ = iterkeys


if __name__ == '__main__':
    import doctest
    doctest.testmod()



import re, datetime, time
from datetime import timedelta

# http://code.activestate.com/recipes/306860-proleptic-gregorian-dates-and-strftime-before-1900/


# remove the unsupposed "%s" command.  But don't
# do it if there's an even number of %s before the s
# because those are all escaped.  Can't simply
# remove the s because the result of
#  %sY
# should be %Y if %s isn't supported, not the
# 4 digit year.
_illegal_s = re.compile(r"((^|[^%])(%%)*%s)")

# Every 28 years the calendar repeats, except through century leap
# years where it's 6 years.  But only if you're using the Gregorian
# calendar.  ;)

def strftime(dt, fmt):
    if _illegal_s.search(fmt):
        raise TypeError("This strftime implementation does not handle %s")
    if dt.year > 1900:
        return dt.strftime(fmt)

    fmt = fmt.replace('%c', '%a %b %d %H:%M:%S %Y')\
             .replace('%Y', str(dt.year))\
             .replace('%y', ("%04d" %dt.year)[-2:])

    year = dt.year
    # For every non-leap year century, advance by
    # 6 years to get into the 28-year repeat cycle
    delta = 2000 - year
    off = 6*(delta // 100 + delta // 400)
    year = year + off

    # Move to around the year 2000
    year = year + ((2000 - year)//28)*28
    timetuple = dt.timetuple()
    return time.strftime(fmt, (year,) + timetuple[1:])



# -*- coding: utf-8 -*-

from os.path import dirname, abspath, join
import logging
logger = logging.getLogger(__name__)

import jinja2

__all__ = ('TemplateEngine', 'TEMPLATE_DIR')

CURDIR = dirname(abspath(__file__))
TEMPLATE_DIR = join(CURDIR, 'templates')


class TemplateEngine(object):
    def __init__(self, paths, cache=False):
        '''
        paths - list of paths
        '''
        self.env = jinja2.Environment(
            loader=jinja2.FileSystemLoader(paths),
            autoescape=True,
        )

    def render(self, template_name, **kw):
        'Interface method'
        return self.env.get_template(template_name).render(**kw)



# -*- coding: utf-8 -*-

class VersionedStorage(object):
    def __init__(self, *args, **kwargs):
        self.__stack = [{}]
        self.__dict__.update(dict(*args, **kwargs))

    def _get_dict(self):
        d = self.__dict__.copy()
        del d['_VersionedStorage__stack']
        return d

    def _set_dict(self, value):
        value['_VersionedStorage__stack'] = self.__stack
        self.__dict__ = value

    _dict_ = property(_get_dict, _set_dict)

    @property
    def _modified(self):
        return self.__stack[-1] != self._dict_

    def _commit(self):
        if self._modified:
            self.__stack.append(self._dict_)

    def _rollback(self):
        if len(self.__stack) > 1:
            self._dict_ = self.__stack.pop()

    def as_dict(self):
        return self._dict_

    def __getitem__(self, k):
        return self.__dict__[k]

    def __delitem__(self, k):
        del self.__dict__[k]

    __setitem__ = object.__setattr__

    def __contains__(self, k):
        return k in self._dict_

    def __repr__(self):
        return repr(self._dict_)

    def __call__(self, **kwargs):
        self.__dict__.update(**kwargs)
        return self



# -*- coding: utf-8 -*-
'''
This module contains basic form access control classes.

You can set permissions to form and fields like this::

    class SampleForm(form.Form):
       permissions = 'rwcx'
       fields=[fields.Field('input',
                            perm_getter=perms.SimplePerm('rw'))]

or pass form permissions to constructor::

    >>> form = SampleForm(env, data, permissions='rw')

To access current permissions set you can use field's :attr:`permissions`
property:
    
    >>> form.get_field('input').permissions
    set(['r', 'w'])

'''

DEFAULT_PERMISSIONS = set('rwc')

class BasePerm(object):
    """
    Permission getters base class.

    Describes the interface for classes, designed for determine permissions of
    different objects, particularly Form Fields.
    """
    
    def get_perms(self, obj):
        '''
        Returns combined Environment's and object's permissions.
        Resulting condition is intersection of them.
        '''
        return self.available(obj) & self.check(obj)

    def available(self, obj):
        '''
        Returns permissions according environment's limiting condition.
        Determined by object's context
        
        Ancestors must override this method
        '''
        return DEFAULT_PERMISSIONS
    
    def check(self, obj):
        '''
        Returns permissions determined by object itself

        Ancestors must override this method
        '''
        return DEFAULT_PERMISSIONS


class FieldPerm(BasePerm):
    '''
    Default permission getter for Field objects
    
    Ancestor should override the :meth:`check` method. They can use field.env
    to get any values from outside. For example::
    
        class RoleBased(FieldPerm):
            def __init__(self, role_perms):
                self.role_perms = role_perms
        
            def check(self, field):
                user = field.env.user
                perms = set(self.role_perms.get('*', ''))
                for role in user.roles:
                    perms.update(self.role_perms.get(role, ''))
                return perms
    '''
    
    def available(self, field):
        '''
        Allows only field's parents' permissions
        '''
        return field.parent.permissions

    def check(self, field):
        '''
        Returns permissions determined by object itself

        Ancestors must override this method
        '''
        return field.parent.permissions
    
    
class SimplePerm(FieldPerm):
    '''
    Permission getter returning determined set of permissions
    '''

    def __init__(self, permissions):
        self.permissions = set(permissions)
        
    def check(self, field):
        return self.permissions


'''
'''


# -*- coding: utf-8 -*-

__all__ = ['Request', 'Response']

import logging
import httplib
import cgi
from webob import Request as _Request, Response
from webob.multidict import MultiDict, NoVars, UnicodeMultiDict

from ..utils import cached_property

logger = logging.getLogger(__name__)



class Request(_Request):
    '''
    Patched webob Request class
    '''

    def __init__(self, *args, **kwargs):
        super(Request, self).__init__(*args, **kwargs)
        self._prefixes = []
        self._subdomain = ''

    def add_prefix(self, prefix):
        self._prefixes.append(prefix)

    def add_subdomain(self, subdomain):
        if self._subdomain and subdomain:
            self._subdomain = subdomain + '.' + self._subdomain
        elif subdomain:
            self._subdomain = subdomain

    # We need to inject code which works with
    # prefixes
    @property
    def prefixed_path(self):
        path = self.path
        if self._prefixes:
            length = sum(map(len, self._prefixes))
            path = path[length:]
        return path

    @property
    def prefixed_path_qs(self):
        path = self.path_qs
        if self._prefixes:
            length = sum(map(len, self._prefixes))
            path = path[length:]
        return path

    @property
    def subdomain(self):
        path = self.server_name.decode('idna')
        if self._subdomain:
            path = path[:-len(self._subdomain)-1]
        return path

    @cached_property
    def FILES(self):
        return self._sub_post(lambda x: isinstance(x[1], cgi.FieldStorage))

    @cached_property
    def POST(self):
        return self._sub_post(lambda x: not isinstance(x[1], cgi.FieldStorage))

    def _sub_post(self, condition):
        post = super(Request, self).str_POST
        if isinstance(post, NoVars):
            return post
        return UnicodeMultiDict(MultiDict(filter(condition, post.items())),
                                encoding=self.charset,
                                errors=self.unicode_errors,
                                decode_keys=self.decode_param_names)



# -*- coding: utf-8 -*-

from time import time
import struct, os, itertools

from webob.multidict import MultiDict
from ..utils import weakproxy, cached_property

from . import convs
from .perms import DEFAULT_PERMISSIONS
from .media import FormMedia


class FormEnvironment(object):
    def __init__(self, **kw):
        self.__dict__.update(kw)


class Form(object):

    template = 'forms/default'
    media = FormMedia()
    permissions = DEFAULT_PERMISSIONS

    def __init__(self, env=None, initial=None, name=None, permissions=None):
        env = env or {}
        initial = initial or {}
        self.env = FormEnvironment(**env) if isinstance(env, dict) else env
        self.name = name
        self.raw_data = raw_data = MultiDict()
        #NOTE: if you provide initial value for some aggregated field
        #      you need to provide values for all fields that are in that
        #      aggregated field, including emty values (None)
        self.initial = initial = initial or {}
        self.python_data = initial.copy()
        # clone all fields
        self.fields = [field(parent=self) for field in self.fields]

        if permissions is None:
            # to allow permissions definition in Form class
            permissions = self.permissions
        self.permissions = set(permissions)

        for field in self.fields:
            if field.name in initial:
                value = initial[field.name]
            else:
                # get_default() may return different values for each call, so
                # we have to insure converted value match python one.
                value = field.get_default()
            self.python_data[field.name] = value
            field.set_raw_value(field.from_python(value))
        self.errors = {}

    @cached_property
    def id(self):
        '''Random ID for given form input'''
        # Time part is repeated in about 3 days period
        time_part = struct.pack('!d', time())[3:]
        return (time_part+os.urandom(1)).encode('hex')

    @property
    def form(self):
        return self

    @property
    def prefix(self):
        '''A prefix for names of field inputs'''
        if self.name:
            return self.name+':'
        else:
            return ''

    def render(self):
        '''Proxy method to form's environment render method'''
        return self.env.template.render(self.template, form=self)

    @property
    def is_valid(self):
        '''Is true if validated form as no errors'''
        return not self.errors

    def get_media(self):
        '''
        Returns a list of FormMedia objects related to the form and
        all of it's fields
        '''
        media = FormMedia(self.media, env=self.env)
        for field in self.fields:
            media += field.get_media()
        return media

    def accept(self, data, files=None):
        self.raw_data = MultiDict(data)
        self.files = files or MultiDict()
        self.errors = {}
        for field in self.fields:
            if field.writable:
                self.python_data[field.name] = field.accept()
            else:
                # readonly field
                field.set_raw_value(field.from_python(self.python_data[field.name]))

        if not self.is_valid:
            return False

        for field in self.fields:
            validate = getattr(self, 'clean__%s' % field.name, None)
            if validate:
                try:
                    self.python_data[field.name] = \
                        validate(self.python_data.get(field.name, None))
                except convs.ValidationError, e:
                    self.errors[field.input_name] = e.message
                    del self.python_data[field.name]

        return self.is_valid

    def get_field(self, name):
        '''
        Gets field by input name
        '''
        names = name.split('.', 1)
        for field in self.fields:
            if field.name == names[0]:
                if len(names) > 1:
                    return field.get_field(names[1])
                return field
        return None



# -*- coding: utf-8 -*-
'''
Module containing classes for easy and flexible HTML cleanup
'''

__all__ = ['Sanitizer', 'sanitize', 'remove_a_tags_without_href', 'strip_empty_tags',
           'strip_empty_tags_nested']

import re
from xml.sax.saxutils import escape, unescape
from html5lib import HTMLParser, sanitizer, treebuilders, treewalkers, serializer
from html5lib.constants import tokenTypes
from html5lib.html5parser import ParseError


SAFE_CLASSES = {}

class TokenSanitazer(sanitizer.HTMLSanitizer):

    escape_invalid_tags = False

    # only html (not SVG or MathML) elements and attributes
    allowed_elements = sanitizer.HTMLSanitizer.acceptable_elements
    allowed_attributes = sanitizer.HTMLSanitizer.acceptable_attributes
    allowed_classes = SAFE_CLASSES

    options = ('allowed_elements', 'allowed_attributes', 'allowed_css_properties',
               'allowed_css_keywords', 'allowed_protocols', 'escape_invalid_tags',
               'allowed_classes', 'attr_val_is_uri')

    def __init__(self, *args, **kwargs):
        for key in kwargs.keys():
            if key in self.options:
                setattr(self, key, kwargs.pop(key))
            elif key not in ('encoding', 'parseMeta', 'useChardet', 
                             'lowercaseElementName', 'lowercaseAttrName'):
                kwargs.pop(key)
        super(TokenSanitazer, self).__init__(*args, **kwargs)

    def sanitize_token(self, token):
        if token["type"] in (tokenTypes["StartTag"], tokenTypes["EndTag"], 
                             tokenTypes["EmptyTag"]):
            if token["name"] in self.allowed_elements:
                if token.has_key("data"):
                    # Copypasted from html5lib
                    attrs = dict([(name,val) for name,val in
                                  token["data"][::-1] 
                                  if name in self.allowed_attributes])
                    for attr in self.attr_val_is_uri:
                        if not attrs.has_key(attr):
                            continue
                        val_unescaped = re.sub("[`\000-\040\177-\240\s]+", '',
                                               unescape(attrs[attr])).lower()
                        #remove replacement characters from unescaped characters
                        val_unescaped = val_unescaped.replace(u"\ufffd", "")
                        if (re.match("^[a-z0-9][-+.a-z0-9]*:",val_unescaped) and
                            (val_unescaped.split(':')[0] not in 
                             self.allowed_protocols)):
                            del attrs[attr]
                    # end copypasted

                    if attrs.has_key('style'):
                        styles = self.sanitize_css(attrs.pop('style'))
                        if styles:
                            attrs['style'] = styles
                    if attrs.has_key('class'):
                        attrs = self.sanitize_classes(token, attrs)
                    token["data"] = [[name,val] for name,val in attrs.items()]
                return token
            elif self.escape_invalid_tags:
                return self.escape_token(token)
        elif token["type"] == tokenTypes["Comment"]:
            pass
        else:
            return token
        
    def sanitize_classes(self, token, attrs):
        # drop restricted classes
        classes = attrs.pop('class').split()
        if token['name'] in self.allowed_classes:
            allowed = self.allowed_classes[token['name']]
            condition = callable(allowed) and allowed or \
                        (lambda cls: cls in allowed)
            value = ' '.join(filter(condition, classes))
            if value:
                attrs['class'] = value
        return attrs
    
    def escape_token(self, token):
        # a part of html5lib sanitize_token method
        if token["type"] == tokenTypes["EndTag"]:
            token["data"] = "</%s>" % token["name"]
        elif token["data"]:
            attrs = ''.join([' %s="%s"' % (k,escape(v)) for k,v in token["data"]])
            token["data"] = "<%s%s>" % (token["name"],attrs)
        else:
            token["data"] = "<%s>" % token["name"]
        if token["type"] == tokenTypes["EmptyTag"]:
            token["data"] = token["data"][:-1] + "/>"
        token["type"] = tokenTypes["Characters"]
        del token["name"]
        return token

def remove_a_tags_without_href(dom_tree, **kwargs):
    a_tags = []
    for node in dom_tree.childNodes:
        if node.nodeType == node.ELEMENT_NODE:
            if node.tagName == 'a':
                a_tags.append(node)
            a_tags.extend(node.getElementsByTagName('a'))
    a_tags = filter(lambda x: not x.hasAttribute('href'), a_tags)

    for a in a_tags:
        while a.childNodes:
            a.parentNode.insertBefore(a.childNodes[0],a)
        a.parentNode.removeChild(a)
        #a.unlink()
    return dom_tree


def strip_empty_tags(clean, drop_empty_tags=[], **kwargs):
    # XXX Sometimes doesn't work (with nested empty tags)
    for tag in drop_empty_tags:
        r = re.compile(r'<%s[^>]*>((\s|&nbsp;)*)<\/%s>' % (tag, tag), re.UNICODE)
        clean = r.sub(r'\1', clean)
    return clean

def strip_empty_tags_nested(clean, drop_empty_tags=[], **kwargs):
    # XXX Ugly but working function
    regs = [re.compile(r'<%s[^>]*>((\s|&nbsp;)*)<\/%s>' % (tag, tag), re.UNICODE)
            for tag in drop_empty_tags]
    subns = 1
    while subns:
        subns = 0
        for r in regs:
            clean, n = r.subn(r'\1', clean)
            subns += n
    return clean

class Sanitizer(object):
    '''
    Sanitizer object customizing sanitarization
    and providing method for sanitizing string.
    Is used by :class:`forms.convs.Html<insanities.forms.convs.Html>`.

    There are some options accepted by constructor:

    * *allowed_elements*. A list of HTML elements tag names
      that are not removed or escaped.
    * *allowed_attributes*. A list of HTML attributes that
      are not removed.
    * *escape_invalid_tags*. If it's True, disallowed HTML
      elements are escaped, otherwise they are removed.
    * *allowed_classes*. Dict with keys representing HTML
      tag names and values describing what classes are allowed
      for this element. It can be list of accepted classes
      (strings) or callable accepting the classname and
      returning condition if it is allowed::

        Sanitizer(allowed_classes={
            'p': ['hidden'],
            'span': re.compile('^num_\d+$').match,
        })
    * *strip_whitespaces*. Strip whitespaces or not.
    * *method*. Method of html rendering, default is xhtml.
      See html5lib docs for details.
    * *dom_callbacks*. A list of functions called after HTML
      is parsed. Function accept and returns minidom
      :class:`DocumentFragment` object and can process it. 
    * *string_callbacks*. A list of functions called after HTML
      is rendered. Function accept and returns the rendered
      HTML string. 

    For more options (including CSS, SVG, MathML cleanup)
    see source code and html5lib documentation.
    '''

    dom_callbacks = [remove_a_tags_without_href]
    string_callbacks = [strip_empty_tags]
    method = 'xhtml'
    strip_whitespace = True
    tokensanitazer = TokenSanitazer

    options = ('dom_callbacks', 'string_callbacks', 'method', 'strip_whitespace')

    def __init__(self, **kwargs):
        for key in kwargs.keys():
            if key in self.options:
                setattr(self, key, kwargs.pop(key))
        self.kwargs = kwargs

    def token_sanitizer(self):
        # Proxy function to pass arguments into Sanitizer constructor
        def func(*args, **kwargs):
            kwargs.update(self.kwargs)
            return self.tokensanitazer(*args, **kwargs)
        return func

    def get_dom(self, buf):
        buf = buf.strip()
        if not buf:
            return None
        p = HTMLParser(tree=treebuilders.getTreeBuilder("dom"),
                                tokenizer=self.token_sanitizer())
        return p.parseFragment(buf)

    def render(self, dom_tree):
        walker = treewalkers.getTreeWalker("dom")
        stream = walker(dom_tree)
        if self.method == "xhtml":
            Serializer = serializer.xhtmlserializer.XHTMLSerializer
        else:
            Serializer = serializer.htmlserializer.HTMLSerializer
        ser = Serializer(strip_whitespace=self.strip_whitespace,
                         quote_attr_values=True,
                         omit_optional_tags=False)
        return ser.render(stream)

    def sanitize(self, htmlstring):
        '''
        HTML sanitirization with html5lib-like interface
        '''
        dom_tree = self.get_dom(htmlstring)
        if dom_tree is None:
            return ''

        for callback in self.dom_callbacks:
            dom_tree = callback(dom_tree, **self.kwargs)

        clean = self.render(dom_tree)

        for callback in self.string_callbacks:
            clean = callback(clean, **self.kwargs)

        return unicode(clean)


def _get_list_props(cls):
    return [(opt, getattr(cls, opt))
            for opt in cls.options
            if type(getattr(cls, opt)) in (list, tuple)]


PROPERTIES = list(TokenSanitazer.options) + list(Sanitizer.options)
LIST_PROPERTIES = dict(_get_list_props(TokenSanitazer) + _get_list_props(Sanitizer))


def sanitize(buf, **kwargs):
    return Sanitizer(**kwargs).sanitize(buf)



# -*- coding: utf-8 -*-

__all__ = ['ask']

from .http import Request
from ..utils.storage import VersionedStorage


def ask(application, url, method='get', data=None,
        headers=None, additional_env=None, additional_data=None):
    env = VersionedStorage(additional_env or {})
    #TODO: may be later process cookies separatly
    env.request = Request.blank(url, POST=data, headers=headers)
    data = VersionedStorage(additional_data or {})
    return application(env, data)



# -*- coding: utf-8 -*-

class MultiDict(dict):
    """
    Dictionary that allows to map multiple ``values``
    to one ``key``.

    >>> md = MultiDict((('foo', 'bar'), ('spam', 'eggs'), ('foo', 'baz')))
    >>> md['foo']
    ['bar', 'baz']
    >>> md['spam']
    ['eggs']
    >>> md.getlist('foo')
    ['bar', 'baz']
    """

    def __init__(self, initial_data={}, *args, **kwargs):
        """
        .. describe:: initial_data

           Iterable with items represented as tuple or list of ``key``, ``value``.
           Or ``dict`` object, or other ``MultiDict`` object.

        >>> list_ = (('foo','bar'),('spam','eggs'),('foo','baz'))
        >>> md = MultiDict(list_)
        >>> print md
        MultiDict([('foo', 'bar'), ('foo', 'baz'), ('spam', 'eggs')])
        >>> new_md = MultiDict(md)
        >>> print new_md
        MultiDict([('foo', 'bar'), ('foo', 'baz'), ('spam', 'eggs')])
        """
        super(MultiDict, self).__init__(*args, **kwargs)

        if hasattr(initial_data, 'items'):
            # Предполагаем, что initial_data - словарь или экземпляр
            # MultiDict. Просто сохранить копию словаря нельзя, потому что
            # внутреннее представление данных - словарь, каждому
            # ключу которого соответствует список значений
            initial_data = initial_data.items()

        for key, value in initial_data:
            self.append(key, value)

    def append(self, key, value):
        """
        Method appends ``value`` for ``key``, if ``key`` doesn't
        exists it will be created.

        >>> md = MultiDict()
        >>> md.append('foo', 'bar')
        >>> md.append('foo', 'baz')
        >>> md.append('spam', 'eggs')
        >>> print md
        MultiDict([('foo', 'bar'), ('foo', 'baz'), ('spam', 'eggs')])
        """
        self.setdefault(key, []).append(value)

    getlist = dict.__getitem__

    def getfirst(self, key):
        """
        Method returns first value from values list
        of the ``key``.

        >>> md = MultiDict((('foo', 'bar'), ('spam', 'eggs'), ('foo', 'baz')))
        >>> md.getfirst('spam')
        'eggs'
        >>> md.getfirst('foo')
        'bar'
        """
        return super(MultiDict, self).__getitem__(key)[0]

    def getlast(self, key):
        """
        Method returns last value from values list
        of the ``key``.

        >>> md = MultiDict((('foo', 'bar'), ('spam', 'eggs'), ('foo', 'baz')))
        >>> md.getlast('spam')
        'eggs'
        >>> md.getlast('foo')
        'baz'
        """
        return super(MultiDict, self).__getitem__(key)[-1]

    def __setitem__(self, key, value):
        """
        Method maps ``value`` to ``key``

        .. describe:: value
           
           ``list`` or ``tuple``

        >>> md = MultiDict()
        >>> md['foo'] =  'bar'# doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        AssertionError: Values must be type list or tuple
        >>> md['foo'] = ['spam', 'eggs']
        >>> print md
        MultiDict([('foo', 'spam'), ('foo', 'eggs')])
        """
        assert isinstance(value, list),\
                "Values must be type list"
        super(MultiDict, self).__setitem__(key, value)

    # Метод вернется, если надумаем делать setone()
    #setlist = __setitem__

    def update(self, items):
        raise NotImplementedError


    def items(self):
        """
        Method returns list of tuples (``key``, ``value``).
        If ``key`` has multiple values, then multiple tuples
        will be returned for this ``key``.

        >>> md = MultiDict((('foo', 'bar'), ('spam', 'eggs'), ('foo', 'baz')))
        >>> md.items()
        [('foo', 'bar'), ('foo', 'baz'), ('spam', 'eggs')]
        """
        return list(self.iteritems())

    def iteritems(self):
        for k,v in super(MultiDict, self).iteritems():
            for item in v:
                yield (k, item)

    def copy(self):
        return self.__class__(self)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.items())



# -*- coding: utf-8 -*-

__all__ = ['match', 'method', 'static_files', 'ctype', 'prefix', 
           'subdomain', 'namespace']

import logging
import httplib
import mimetypes
from os import path
from urllib import unquote
from .core import WebHandler
from .http import Response
from .url import UrlTemplate


logger = logging.getLogger(__name__)


def update_data(data, new_data):
    for k,v in new_data.items():
        data[k] = v


class match(WebHandler):

    def __init__(self, url, name, convs=None):
        self.url = url
        self.url_name = name
        self.builder = UrlTemplate(url, converters=convs)

    def _locations(self):
        return {self.url_name: {'builders': [self.builder]}}

    def handle(self, env, data, next_handler):
        matched, kwargs = self.builder.match(env.request.prefixed_path, env=env)
        if matched:
            env.current_url_name = self.url_name
            update_data(data, kwargs)
            return next_handler(env, data)
        return None

    def __repr__(self):
        return '%s(\'%s\', \'%s\')' % \
                (self.__class__.__name__, self.url, self.url_name)


class method(WebHandler):
    def __init__(self, *names):
        self._names = [name.upper() for name in names]

    def handle(self, env, data, next_handler):
        if env.request.method in self._names:
            return next_handler(env, data)
        return None

    def __repr__(self):
        return 'method(*%r)' % self._names


class ctype(WebHandler):

    xml = 'application/xml'
    json = 'application/json'
    html = 'text/html'
    xhtml = 'application/xhtml+xml'

    def __init__(self, *types):
        self._types = types

    def handle(self, env, data, next_handler):
        if env.request.content_type in self._types:
            return next_handler(env, data)
        return None

    def __repr__(self):
        return '%s(*%r)' % (self.__class__.__name__, self._types)


class static_files(WebHandler):
    def __init__(self, location, url='/static/'):
        self.location = location
        self.url = url

    def construct_reverse(self):
        def url_for_static(part):
            while part.startswith('/'):
                part = part[1:]
            return path.join(self.url, part)
        return url_for_static

    def handle(self, env, data, next_handler):
        path_info = unquote(env.request.path)
        if path_info.startswith(self.url):
            static_path = path_info[len(self.url):]
            while static_path[0] in ('.', '/', '~'):
                static_path = static_path[1:]
            file_path = path.join(self.location, static_path)
            if path.exists(file_path) and path.isfile(file_path):
                mime = mimetypes.guess_type(file_path)[0]
                response = Response()
                if mime:
                    response.content_type = mime
                with open(file_path, 'r') as f:
                    response.write(f.read())
                return response
            else:
                logger.info('Client requested non existent static data "%s"' % file_path)
                return Response(status=404)
        return None


class prefix(WebHandler):
    def __init__(self, _prefix, convs=None):
        self.builder = UrlTemplate(_prefix, match_whole_str=False, 
                                   converters=convs)

    def _locations(self):
        locations = super(prefix, self)._locations()
        for v in locations.values():
            v.setdefault('builders', []).append(self.builder)
        return locations

    def handle(self, env, data, next_handler):
        matched, kwargs = self.builder.match(env.request.prefixed_path, env=env)
        if matched:
            update_data(data, kwargs)
            env.request.add_prefix(self.builder(**kwargs))
            return next_handler(env, data)
        return None

    def __repr__(self):
        return '%s(\'%r\')' % (self.__class__.__name__, self.builder)


class subdomain(WebHandler):
    def __init__(self, _subdomain):
        self.subdomain = unicode(_subdomain)

    def _locations(self):
        locations = super(subdomain, self)._locations()
        for v in locations.values():
            v.setdefault('subdomains', []).append(self.subdomain)
        return locations

    def handle(self, env, data, next_handler):
        subdomain = env.request.subdomain
        #XXX: here we can get 'idna' encoded sequence, that is the bug
        if self.subdomain:
            slen = len(self.subdomain)
            delimiter = subdomain[-slen-1:-slen]
            matches = subdomain.endswith(self.subdomain) and delimiter in ('', '.')
        else:
            matches = not subdomain

        if matches:
            env.request.add_subdomain(self.subdomain)
            return next_handler(env, data)
        return None

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.subdomain)


class namespace(WebHandler):
    def __init__(self, ns):
        # namespace is str
        self.namespace = ns

    def handle(self, env, data, next_handler):
        if 'namespace' in env:
            env.namespace += '.' + self.namespace
        else:
            env.namespace = self.namespace
        return next_handler(env, data)

    def _locations(self):
        locations = super(namespace, self)._locations()
        new_locations = {}
        for k, v in locations.items():
            new_locations[self.namespace+'.'+k if k else self.namespace] = v
        return new_locations



# -*- coding: utf-8 -*-

from ..utils import cached_property


class FormMedia(object):
    '''
    Instance encapsulating media items assigned to instances of
    :class:`Form <insanities.forms.form.Form>`,
    :class:`Field <insanities.forms.fields.Field>` and
    :class:`Widget <insanities.forms.widgets.Widget>`
    '''

    def __init__(self, items=None, env=None):
        self.env = env
        self._media = []
        map(self._append, items or [])

    def _append(self, item):
        if item not in self._media:
            self._media.append(item(holder=self))

    def __iadd__(self, other):
        # `other` is iterable (including FormMedia)
        map(self._append, other)
        return self

    def __add__(self, other):
        media = FormMedia(self)
        media += other
        return media

    def __radd__(self, other):
        return self.__add__(other)

    def __iter__(self):
        return iter(self._media)

    def __nonzero__(self):
        return bool(self._media)

    def __eq__(self, other):
        return self._media == other._media

    def __repr__(self):
        return '%s(items=%r)' % (self.__class__.__name__, self._media)


class FormMediaAtom(object):
    '''
    Media item representing JS, CSS or some other media stuff linked to
    :class:`Form <insanities.forms.form.Form>` or it's parts.
    '''

    macro = None # Must be overwritten

    def __init__(self, data, holder=None):
        self.data = data
        self.holder = holder

    def __call__(self, holder):
        return type(self)(self.data, holder=holder)

    def __eq__(self, other):
        return type(self)==type(other) and self.data==other.data

    def __iter__(self):
        yield self

    def __add__(self, other):
        media = FormMedia(self)
        media += other
        return media

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.data)

    def render(self):
        '''Renders media item to HTML'''
        return self.holder.env.macros[self.macro](data=self.data, 
                                                  env=self.holder.env)


class FormCSSRef(FormMediaAtom):

    macro = 'css_ref'


class FormCSSInline(FormMediaAtom):

    macro = 'css_inline'


class FormJSRef(FormMediaAtom):

    macro = 'js_ref'


class FormJSInline(FormMediaAtom):

    macro = 'js_inline'



# -*- coding: utf-8 -*-

__all__ = ['WebHandler', 'cases', 'handler', 'Reverse',
           'locations']

import logging
import types
import httplib
from webob.exc import HTTPException
from .http import Request, Response
from ..utils.storage import VersionedStorage
from .url import URL



logger = logging.getLogger(__name__)


def prepare_handler(handler):
    '''Wrapps functions, that they can be usual WebHandler's'''
    if type(handler) in (types.FunctionType, types.MethodType):
        handler = FunctionWrapper(handler)
    return handler


class WebHandler(object):
    '''Base class for all request handlers.'''

    def __or__(self, next_handler):
        if hasattr(self, '_next_handler'):
            self._next_handler | next_handler
        else:
            self._next_handler = prepare_handler(next_handler)
        return self

    def handle(self, env, data, next_handler):
        '''This method should be overridden in subclasses.'''
        return next_handler(env, data)

    def _locations(self):
        next_handler = self.get_next()
        # if next_handler is lambda - the end of chain
        if not type(next_handler) is types.FunctionType:
            return next_handler._locations()
        # we are last in chain
        return {}

    def __repr__(self):
        return '%s()' % self.__class__.__name__

    def __call__(self, env, data):
        next_handler = self.get_next()
        env._commit()
        data._commit()
        result = self.handle(env, data, next_handler)
        if result is None:
            if env._modified:
                env._rollback()
            if data._modified:
                data._rollback()
        return result

    def get_next(self):
        if hasattr(self, '_next_handler'):
            return self._next_handler
        #XXX: may be FunctionWrapper?
        return lambda e, d: None

    def as_wsgi(self):
        def wrapper(environ, start_response):
            env = VersionedStorage()
            env.request = Request(environ, charset='utf-8')
            data = VersionedStorage()
            try:
                response = self(env, data)
                if response is None:
                    logger.debug('Application returned None '
                                 'instead of Response object')
                    status_int = httplib.NOT_FOUND
                    response = Response(status=status_int, 
                                        body='%d %s' % (status_int, 
                                                        httplib.responses[status_int]))
            except HTTPException, e:
                response = e
            except Exception, e:
                logger.exception(e)
                raise

            headers = response.headers.items()
            start_response(response.status, headers)
            return [response.body]
        return wrapper


class Reverse(object):

    def __init__(self, urls, env=None):
        self.urls = urls
        self.env = env

    @property
    def namespace(self):
        if self.env and 'namespace' in self.env:
            return self.env.namespace
        return ''

    def __call__(self, name, **kwargs):
        if name.startswith('.'):
            local_name = name.lstrip('.')
            up = len(name) - len(local_name) - 1
            if up != 0:
                ns = ''.join(self.namespace.split('.')[:-up])
            else:
                ns = self.namespace
            name = ns + '.' + local_name

        data = self.urls[name]

        host = u'.'.join(data.get('subdomains', []))
        # path - urlencoded str
        path = ''.join([b(**kwargs) for b in reversed(data['builders'])])
        return URL(path, host=host)

    @classmethod
    def from_handler(cls, handler, env=None):
        return cls(locations(handler), env=env)


class cases(WebHandler):

    def __init__(self, *handlers, **kwargs):
        self.handlers = []
        for handler in handlers:
            self.handlers.append(prepare_handler(handler))

    def __or__(self, next_handler):
        'cases needs to set next handler for each handler it keeps'
        for handler in self.handlers:
            handler | prepare_handler(next_handler)
        return self

    def handle(self, env, data, next_handler):
        for handler in self.handlers:
            result = handler(env, data)
            if result is None:
                continue
            return result

    def _locations(self):
        locations = {}
        for handler in self.handlers:
            handler_locations = handler._locations()
            for k, v in handler_locations.items():
                if k in locations:
                    raise ValueError('Location "%s" already exists' % k)
                locations[k] = v
        return locations

    def __repr__(self):
        return '%s(*%r)' % (self.__class__.__name__, self.handlers)


class FunctionWrapper(WebHandler):
    '''Wrapper for handler represented by function'''

    def __init__(self, func):
        self.func = func

    def handle(self, env, data, next_handler):
        return self.func(env, data, next_handler)

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self.func.__name__)


handler = FunctionWrapper


def locations(handler):
    return handler._locations()



# -*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='insanities',
    version='0.3',
    packages=['insanities', 'insanities.utils', 'insanities.forms',
              'insanities.web', 'insanities.templates', 'insanities.templates.mint',
              'insanities.templates.jinja2'],
    package_dir={
        'insanities.templates.jinja2':'insanities/templates/jinja2',
        'insanities.templates.mint':'insanities/templates/mint',
    },
    package_data={
        'insanities.templates.jinja2':['templates/*/*.html'],
        'insanities.templates.mint':['templates/*/*.mint'],
    },
    requires=[
        'webob',
    ],
    author='Denis Otkidach',
    author_email='denis.otkidach@gmail.com',
    maintainer='Tim Perevezentsev',
    maintainer_email='riffm2005@gmail.com',
    description='A set of insanities of several geeks to show ourselves our coolness.',
    long_description=open('README').read(),
    url='http://github.com/riffm/insanities-testing/',
    license='MIT',
    keywords='web forms',
)



# -*- coding: utf-8 -*-

from copy import deepcopy
from ..utils import weakproxy, cached_property
from . import convs
from .media import FormMedia, FormCSSRef, FormJSRef

class Widget(object):

    #: Template to render widget
    template = None
    #: List of :class:`FormMediaAtom<insanities.forms.media.FormMediaAtom>`
    #: objects associated with the widget
    media = []
    #: Value of HTML element's *class* attribute
    classname = ''

    def __init__(self, field=None, **kwargs):
        self.field = weakproxy(field)
        self._init_kwargs = kwargs
        self.__dict__.update(kwargs)

    @property
    def multiple(self):
        return self.field.multiple

    @property
    def input_name(self):
        return self.field.input_name

    @property
    def id(self):
        return self.field.id

    @property
    def env(self):
        return self.field.env

    def get_media(self):
        return FormMedia(self.media)

    def prepare_data(self, value):
        '''
        Method returning data passed to template.
        Subclasses can override it.
        '''
        return dict(widget=self,
                    value=value,
                    readonly=not self.field.writable)

    def render(self, value):
        '''
        Renders widget to template
        '''
        data = self.prepare_data(value)
        if self.field.readable:
            return self.env.template.render(self.template, **data)
        return ''

    def __call__(self, **kwargs):
        kwargs = dict(self._init_kwargs, **kwargs)
        kwargs.setdefault('field', self.field)
        return self.__class__(**kwargs)


class TextInput(Widget):

    template = 'widgets/textinput'
    classname = 'textinput'


class HiddenInput(Widget):

    template = 'widgets/hiddeninput'


class PasswordInput(Widget):

    template = 'widgets/passwordinput'
    classname = 'textinput'


class Select(Widget):
    '''
    Takes options from :class:`EnumChoice<EnumChoice>` converter,
    looks up if converter allows null and passed this value as template
    :obj:`required` variable.
    '''
    template = 'widgets/select'
    classname = 'select'
    #: HTML select element's select attribute value.
    size = None
    #: Label assigned to None value if field is not required
    null_label = '--------'

    def get_options(self, value):
        options = []
        if not self.multiple and (value is None or not self.field.conv.required):
            options = [{'value': '',
                        'title': self.null_label,
                        'selected': value in (None, '')}]
        assert isinstance(self.field.conv, convs.EnumChoice)

        values = value if self.multiple else [value]
        values = map(unicode, values)
        for choice, label in self.field.conv:
            choice = unicode(choice)
            options.append(dict(value=choice,
                                title=label,
                                selected=(choice in values)))
        return options

    def prepare_data(self, value):
        data = Widget.prepare_data(self, value)
        return dict(data,
                    options=self.get_options(value),
                    required=('true' if self.field.conv.required else 'false'))


class GroupedSelect(Select):

    template = 'widgets/grouped_select'
    classname = 'grouped_select select'
    size = None

    def get_options(self, value):
        assert isinstance(self.field.conv, convs.EnumChoice)
        options = []
        if not self.multiple and (value is None or not self.field.conv.required):
            options = [dict(value='', title=self.null_label,
                            selected=value in (None, ''),
                            is_group=False)]
        values = value if self.multiple else [value]
        values = map(unicode, values)

        # TODO fix tree generation
        _group_items = []
        _group_name = None
        for group, choice, label in self.field.conv:
            choice = unicode(choice)
            if (not group and _group_name) or (_group_name and _group_name != group):
                options.append(dict(is_group=True,
                                    title=_group_name,
                                    options=_group_items[0:]))
                _group_name = None
            if group and group != _group_name:
                _group_name = group
                _group_items = []
            if group:
                _group_items.append(dict(value=choice,
                                         title=label,
                                         selected=(choice in values)))
            else:
                options.append(dict(value=choice,
                                    title=label,
                                    selected=(choice in values),
                                    is_group=False))
        if _group_name:
            options.append(dict(is_group=True,
                                title=_group_name,
                                options=_group_items[0:]))
        return options



class CheckBoxSelect(Select):

    template = 'widgets/select-checkbox'


class CheckBox(Widget):

    template = 'widgets/checkbox'


class Textarea(Widget):

    template = 'widgets/textarea'


class TinyMce(Widget):

    template = 'widgets/tinymce'

    media = [FormJSRef('tiny_mce/tiny_mce_init.js')]

    #: List of buttons used on widget
    buttons = (('bold', 'italic', 'underline'),
               ('bullist', 'numlist'),
               ('sub', 'sup'),
               ('indent', 'outdent'),
               ('link', 'unlink'),
               ('undo', 'redo'),
               ('fullscreen', ))

    #: List of attached plugins
    plugins = ('safari', 'directionality',
               'fullscreen', 'xhtmlxtras', 'inlinepopups')

    #: Need to be documented
    content_css = None

    #: Need to be documented
    browsers = ('safari', 'gecko', 'msie')

    #: TinyMce initial config
    cfg = {
        'mode': 'exact',
        'elements': '%(name)s',
        'theme': 'advanced',
        'browsers': '%(browsers)s',
        'language': 'ru',
        'plugins': '%(plugins)s',
        'height': 200,
        'cleanup': True,
        'valid_elements': '%(tags)s',
        'inline_styles': False,
        'gecko_spellcheck': True,
        'force_p_newlines' : True,
        'paste_auto_cleanup_on_paste': True,
        'remove_redundant_brs': True,
        'fix_table_elements': True,
        'fix_nesting': True,
        'theme_advanced_buttons1': '%(buttons)s',
        'theme_advanced_buttons2': '',
        'theme_advanced_buttons3': '',
        'theme_advanced_buttons4': '',
        'theme_advanced_toolbar_location': 'top',
        'theme_advanced_toolbar_align': 'left',
        'theme_advanced_statusbar_location': 'bottom',
        'theme_advanced_resizing': True,
    }

    #: Need to be documented
    compress = False

    def select_value(self, value, default):
        if value is not None:
            if callable(value):
                return value()
            if type(value) is dict:
                return value.copy()
            return value
        return default

    def __init__(self, add_plugins=None, add_buttons=None, drop_buttons=None,
                 **kwargs):

        for option in 'buttons', 'plugins', 'browsers', 'cfg', 'content_css':
            val = kwargs.get(option, None)
            default = getattr(self, option)
            kwargs[option] = self.select_value(val, default)

        if add_plugins:
            kwargs['plugins'] = tuple(list(kwargs['plugins']) + list(add_plugins))
        if add_buttons or drop_buttons:
            btns = map(list, kwargs['buttons'])
            if add_buttons:
                for row, buttons in add_buttons.items():
                    btns[row].extend(buttons)
            if drop_buttons:
                btns = map(lambda row: filter(lambda b: b not in drop_buttons, row), btns)
                btns = filter(None, btns)
            kwargs['buttons'] = tuple(map(tuple, btns))

        super(TinyMce, self).__init__(**kwargs)

    @cached_property
    def js_config(self):
        '''Serializes all TinyMce configs into JSON object'''
        buttons = ',|,'.join([','.join(pack) for pack in self.buttons])
        plugins = ','.join(self.plugins)
        browsers = ','.join(self.browsers)
        tags = ''
        if hasattr(self.field.conv, 'allowed_attributes') and \
        hasattr(self.field.conv, 'allowed_elements'):
            tags = '@[%s],%s' % ('|'.join(set(self.field.conv.allowed_attributes)),
                                 ','.join(self.field.conv.allowed_elements))
        cfg = self.cfg.copy()
        for key, value in cfg.items():
            if type(value) is str and '%' in value:
                cfg[key] = value % {'buttons': buttons, 'plugins': plugins,
                                    'browsers': browsers, 'tags': tags,
                                    'name': self.id}
        if self.content_css:
            css = self.content_css
            if not css.startswith('/'):
                css = self.env.cfg.STATIC_URL + css
            cfg['content_css'] = css

        cfg['field_name'] = self.field.resolve_name()

        import json
        return json.dumps(cfg)

    def prepare_data(self, value):
        data = Widget.prepare_data(self, value)
        return dict(data,
                    config=self.js_config,
                    plugins=','.join(self.plugins))



class ReadonlySelect(Select):

    template = 'widgets/readonlyselect'


class CharDisplay(Widget):

    template = 'widgets/span'
    classname = 'chardisplay'
    #: If is True, value is escaped while rendering. 
    #: Passed to template as :obj:`should_escape` variable.
    escape = False
    #: Function converting the value to string.
    getter = staticmethod(lambda v: v)

    def prepare_data(self, value):
        data = Widget.prepare_data(self, value)
        return dict(data,
                    value=self.getter(value),
                    should_escape=self.escape)


class ImageView(Widget):

    template = 'widgets/imageview'
    classname = 'imageview'


class FileInput(Widget):
    '''
    '''
    template = 'widgets/fileinput'

    def prepare_data(self, value):
        data = Widget.prepare_data(self, value)

        field = self.field
        value = field.parent.python_data.get(field.name, None)
        delete = field.form.data.get(field.input_name + '__delete', False)
        if value is None:
            value = field.parent.initial.get(field.name, None)
            if isinstance(value, field.stored_file_cls):
                mode = 'existing'
            else:
                value = None
                mode = 'empty'
        elif isinstance(value, field.stored_file_cls):
            mode = 'existing'
        elif isinstance(value, field.temp_file_cls):
            mode = 'temp'
        else:
            assert None
        return dict(data, value=value, mode=mode, input_name=self.input_name,
                    delete=delete, temp_url=self.env.rctx.conf.temp_url,
                    null=field.null)

class ImageInput(FileInput):
    template = 'widgets/imageinput'




# -*- coding: utf-8 -*-

import re, logging

import widgets
from . import convs
from ..utils import weakproxy, cached_property
from ..utils.odict import OrderedDict
from .perms import FieldPerm
from .media import FormMedia

logger = logging.getLogger(__name__)


class BaseField(object):
    '''
    Simple container class which ancestors represents various parts of Form.

    Encapsulates converter, various fields attributes, methods for data 
    access control
    '''

    #: :class:`FieldPerm` instance determining field's access permissions.
    #: Can be set by field inheritance or throught constructor.
    perm_getter = FieldPerm()

    # defaults
    conv = convs.Char
    widget = widgets.TextInput()
    label = None
    media = FormMedia()

    def __init__(self, name, conv=None, parent=None, **kwargs):
        kwargs.update(dict(
            parent=parent,
            name=name,
            conv=(conv or self.conv)(field=self),
            widget=(kwargs.get('widget') or self.widget)(field=self),
        ))
        self._init_kwargs = kwargs
        self.__dict__.update(kwargs)

    def __call__(self, **kwargs):
        '''
        Creates current object's copy with extra constructor arguments passed.
        '''
        params = dict(self._init_kwargs, **kwargs)
        return self.__class__(**params)

    @property
    def multiple(self):
        return self.conv.multiple

    @property
    def env(self):
        return self.parent.env

    @property
    def form(self):
        return self.parent.form

    @property
    def input_name(self):
        '''
        Name of field's input element generated in account to possible
        nesting of fields. The input name is to be used in templates as value
        of Input (Select, etc) element's Name attribute and Label element's For
        attribute.
        '''
        return self.parent.prefix + self.name

    @property
    def error(self):
        return self.form.errors.get(self.input_name)

    @property
    def clean_value(self):
        '''
        Current field's converted value from form's python_data.
        '''
        return self.parent.python_data[self.name]

    @cached_property
    def _relative_id(self): # XXX what is this?
        return self.form.get_field_id(self)

    @property
    def id(self):
        # We use template names in list to replace, so we must use it here to
        # insure unique IDs.
        return '%s-%s' % (self.form.id, self.input_name)

    def to_python(self, value):
        return self.conv.to_python(value)

    def from_python(self, value):
        return self.conv.from_python(value)

    @cached_property
    def permissions(self):
        '''
        Returns field's access permissions
        '''
        return self.perm_getter.get_perms(self)

    @cached_property
    def writable(self):
        return 'w' in self.permissions

    @cached_property
    def readable(self):
        return 'r' in self.permissions

    def render(self):
        return self.widget.render(self.raw_value)

    def get_media(self):
        media = FormMedia(self.media)
        media += self.widget.get_media()
        return media


class Field(BaseField):
    '''
    Atomic field
    '''

    #: :class:`Conv` subclass or instance used to convert field data 
    #: and validate it
    conv = convs.Char

    def get_default(self):
        if hasattr(self, 'default'):
            return self.default
        if self.multiple:
            return []
        return None

    @property
    def raw_value(self):
        if self.multiple:
            return self.form.raw_data.getall(self.input_name)
        else:
            return self.form.raw_data.get(self.input_name, '')

    def set_raw_value(self, value):
        raw_data = self.form.raw_data
        if self.multiple:
            try:
                del raw_data[self.input_name]
            except KeyError:
                pass
            for v in value:
                raw_data.add(self.input_name, v)
        else:
            raw_data[self.input_name] = value

    def accept(self):
        return self.to_python(self.raw_value)


class AggregateField(BaseField):

    @property
    def python_data(self):
        '''Representation of aggregate value as dictionary.'''
        try:
            value = self.clean_value
        except LookupError:
            value = self.get_default()
        return self.from_python(value)


class FieldSet(AggregateField):
    '''
    Container field aggregating a couple of other different fields
    '''
    template = 'widgets/fieldset'
    def __init__(self, name, conv=convs.Converter, fields=[], **kwargs):
        if kwargs.get('parent'):
            conv = conv(field=self)
            fields = [field(parent=self) for field in fields]
        kwargs.update(dict(
            name=name,
            conv=conv,
            fields=fields,
        ))
        BaseField.__init__(self, **kwargs)

    @property
    def prefix(self):
        return self.input_name+'.'

    def get_field(self, name):
        names = name.split('.', 1)
        for field in self.fields:
            if field.name == names[0]:
                if len(names) > 1:
                    return field.get_field(names[1])
                return field
        return None

    def get_default(self):
        result = dict((field.name, field.get_default())
                      for field in self.fields)
        return self.to_python(result)

    def set_raw_value(self, value):
        # fills in raw_data multidict, resulting keys are field's absolute names
        assert isinstance(value, dict), 'To set raw value need dict, got %r' % value
        for field in self.fields:
            subvalue = value[field.name]
            field.set_raw_value(field.from_python(subvalue))

    def accept(self):
        result = self.python_data
        for field in self.fields:
            if field.writable:
                result[field.name] = field.accept()
            else:
                # readonly field
                field.set_raw_value(field.from_python(result[field.name]))
        return self.to_python(result)

    def render(self):
        return self.env.template.render(self.template, field=self)

    def get_media(self):
        media = BaseField.get_media(self)
        for field in self.fields:
            media += field.get_media()
        return media


class FieldList(AggregateField):
    '''
    Container aggregating a couple of similar fields
    '''

    order = False

    def __init__(self, name, conv=convs.List, field=Field(None),
                 parent=None, **kwargs):
        if parent:
            conv = conv(field=self)
            field = field(parent=self)
        kwargs.update(dict(
            parent=parent,
            name=name,
            conv=conv,
            field=field,
        ))
        BaseField.__init__(self, **kwargs)

    @property
    def prefix(self):
        return self.input_name+'-'

    def get_default(self):
        return []

    def get_field(self, name):
        names = name.split('.', 1)
        if self.field.name == names[0] or self.field.name is None:
            if len(names) > 1:
                return self.field.get_field(names[1])
            return self.field
        return None

    @property
    def indeces_input_name(self):
        return self.input_name+'-indeces'

    def accept(self):
        old = self.python_data
        result = OrderedDict()
        for index in self.form.raw_data.getall(self.indeces_input_name):
            try:
                #XXX: we do not convert index to int, just check it.
                #     is it good idea?
                int(index)
            except ValueError:
                logger.warning('Got incorrect index from form: %r', index)
                continue
            #TODO: describe this
            field = self.field(name=str(index))
            if not field.writable:
                # readonly field
                if index in old:
                    result[field.name] = old[field.name]
            else:
                result[field.name] = field.accept()
        return self.to_python(result)

    def set_raw_value(self, value):
        indeces = []
        for index in range(1, len(value)+1):
            index = str(index)
            subvalue = value[index]
            subfield = self.field(name=index)
            subfield.set_raw_value(subfield.from_python(subvalue))
            indeces.append(index)
        for index in indeces:
            self.form.raw_data.add(self.indeces_input_name, index)

    def render(self):
        return self.env.template.render(self.template, field=self)

    def get_media(self):
        media = BaseField.get_media(self)
        media += self.field.get_media()
        return media



# -*- coding: utf-8 -*-

import os
import logging
logger = logging.getLogger(__name__)
from glob import glob
from ..web import Response

__all__ = ('Template',)


class TemplateError(Exception): pass


class Template(object):
    def __init__(self, *dirs, **kwargs):
        self.globs = kwargs.get('globs', {})
        self.cache = kwargs.get('cache', False)
        self.dirs = []
        for d in dirs:
            self.dirs.append(d)
        self.engines = {}
        for template_type, engine_class in kwargs.get('engines', {}).items():
            self.engines[template_type] = engine_class(self.dirs[:], cache=self.cache)

    def render(self, template_name, **kw):
        logger.debug('Rendering template "%s"' % template_name)
        vars = self.globs.copy()
        vars.update(kw)
        resolved_name, engine = self.resolve(template_name)
        return engine.render(resolved_name, **vars)

    def resolve(self, template_name):
        pattern = template_name
        if not os.path.splitext(template_name)[1]:
            pattern += '.*'
        for d in self.dirs:
            path = os.path.join(d, pattern)
            for file_name in glob(path):
                name, ext = os.path.splitext(file_name)
                template_type = ext[1:]
                if template_type in self.engines:
                    return file_name[len(d)+1:], self.engines[template_type]
        raise TemplateError('Template or engine for template "%s" not found. Dirs %r' % \
                            (pattern, self.dirs))

    def render_to(self, template_name):
        'WebHandler'
        def wrapper(env, data, next_handler):
            data.env = env
            return Response(self.render(template_name, **data.as_dict()))
        return wrapper

    def render_to_response(self, template_name, data, content_type='text/html'):
        'handy method'
        return Response(self.render(template_name, **data),
                        content_type=content_type)



# -*- coding: utf-8 -*-
'''
Module containing some predefined form converters - objects designed to
convart and validate form field's data.
'''

import re
from ..utils import weakproxy, replace_nontext
from datetime import datetime
from ..utils.odict import OrderedDict
from ..utils.dt import strftime
from ..utils import N_, M_


class NotSubmitted(Exception): pass


class SkipReadonly(NotSubmitted): pass


class NestedError(NotSubmitted): pass


class ValidationError(Exception):

    @property
    def message(self):
        return self.args[0]

    def __repr__(self):
        return self.messages.encode('utf-8')


class Converter(object):
    '''
    Base converter with chaining support
    extend this class in order to get custom
    converter.

    Converting:

    :meth:`to_python` method takes value from form
    and converts it to python type

    :meth:`from_python` method takes value as python
    object and converts it to string or something
    else widget can display

    Chaining:

    Result of first converter is passed as input value
    to second. for example::

        convs.Char(max_length=2)|convs.Int()

    will be a convertor which first validates
    if string's length is 2 or less and after
    that converts it to integer

    Error messages redefinition:

    Set error_<type> parameter to your own message template, for example::

        convs.Char(min_length=5,
                   error_min_length='At least %(min_length)s characters required')`
    '''

    required = True

    #: Values are not accepted by Required validator
    error_required = N_('required field')

    def __init__(self, *args, **kwargs):
        self.field = weakproxy(kwargs.get('field'))
        self._init_kwargs = kwargs
        self.__dict__.update(kwargs)
        self.validators_and_filters = args
        self.to_python = self._check(self.to_python)

    # It is defined as read-only property to avoid setting it to True where
    # converter doesn't support it.
    @property
    def multiple(self):
        '''
        Signs if converter is multiple or not.
        Multiple converters usually accept and return collections.
        '''
        return False

    @property
    def env(self):
        return self.field.env

    def _is_empty(self, value):
        return value in ('', [])

    def _check(self, method):
        def wrapper(value, **kwargs):
            field, form = self.field, self.field.form
            if self.required and self._is_empty(value):
                form.errors[self.field.input_name] = self.error_required
                return self.field.parent.python_data[field.name]
            try:
                value = method(value, **kwargs)
                for v in self.validators_and_filters:
                    value = v(value)
            except ValidationError, e:
                form.errors[field.input_name] = e.message
                #NOTE: by default value for field is in python_data,
                #      but this is not true for FieldList where data
                #      is dynamic, so we set value to None for absent value.
                value = field.parent.python_data.get(field.name)
            return value
        return wrapper

    def to_python(self, value):
        """ custom converters should override this """
        if value == '':
            return None
        return value

    def from_python(self, value):
        """ custom converters should override this """
        if value is None:
            value = ''
        return value

    def __call__(self, **kwargs):
        kwargs = dict(self._init_kwargs, **kwargs)
        kwargs.setdefault('field', self.field)
        return self.__class__(*self.validators_and_filters, **kwargs)

    def assert_(self, expression, msg):
        'Shortcut for assertions of certain type'
        if not expression:
            raise ValidationError(msg)


class validator(object):
    'Function decorator'
    def __init__(self, message):
        self.message = message
    def __call__(self, func):
        def wrapper(value):
            if not func(value):
                raise ValidationError(self.message)
            return value
        return wrapper

# Some useful validators

def limit(min_length, max_length):
    'Sting length constraint'
    message = N_('length should be between %(min)d and %(max)d symbols') % dict(min=min_length, max=max_length)

    @validator(message)
    def wrapper(value):
        if not value:
            # it meens that this value is not required
            return True
        if len(value) < min_length:
            return False
        if len(value) > max_length:
            return False
        return True
    return wrapper


def num_limit(min_value, max_value):
    'Numerical values limit'
    message = N_('value should be between %(min)d and %(max)d') % dict(min=min_value, max=max_value)

    @validator(message)
    def wrapper(value):
        if not value:
            # it meens that this value is not required
            return True
        if value < min_value:
            return False
        if value > max_value:
            return False
        return True
    return wrapper


def length(*args):
    'Exact string lengths'
    @validator(u'Length of value is limited to ' + ','.join([str(a) for a in args]))
    def wrapper(value):
        if not value:
            return True
        if not len(str(value)) in args:
            return False
        return True
    return wrapper


@validator(u'Value must be positive')
def positive_num(value):
    if not value:
        return True
    return value > 0


class Char(Converter):

    #: Regexp to match input string
    regex = None
    nontext_replacement = u'\uFFFD' # Set None to disable and empty string to
                                    # remove.
    strip=False

    error_regex = N_('field should match %(regex)s')

    def clean_value(self, value):
        '''
        Additional clean action to preprocess value before :meth:`to_python`
        method.

        Subclasses may define own clean_value method to allow additional clean
        actions like html cleanup, etc.
        '''
        # We have to clean before checking min/max length. It's done in
        # separate method to allow additional clean action in subclasses.
        if self.strip:
            value = value.strip()
        return value

    def to_python(self, value):
        # converting
        value = self.clean_value(value)
        if self.regex:
            regex = self.regex
            if isinstance(self.regex, basestring):
                regex = re.compile(self.regex, re.U)
            if not regex.match(value):
                raise ValidationError(self.error_regex)
        return value

    def from_python(self, value):
        if value is None:
            return ''
        return unicode(value)


class Int(Converter):
    """
    integer converter with max and min values support
    """

    error_notvalid = N_('it is not valid integer')

    def to_python(self, value):
        if value == '':
            return None
        try:
            value = int(value)
        except ValueError:
            raise ValidationError(self.error_notvalid)
        return value

    def from_python(self, value):
        if value is None:
            return ''
        return unicode(value)


class Bool(Converter):

    required = False

    def to_python(self, value):
        return bool(value)

    def from_python(self, value):
        if value:
            return 'checked'
        return ''


class DisplayOnly(Converter):

    def from_python(self, value):
        return value

    def to_python(self, value):
        raise SkipReadonly


class EnumChoice(Converter):
    '''In addition to Converter interface it must provide methods __iter__ and
    get_label.'''

    conv = Char()
    # choices: [(python_value, label), ...]
    choices = ()
    multiple = False
    error_required = N_('you must select a value')

    def from_python(self, value):
        conv = self.conv(field=self.field)
        if self.multiple:
            return [conv.from_python(item) for item in value or []]
        else:
            return conv.from_python(value)

    def _safe_to_python(self, value):
        conv = self.conv(field=self.field)
        try:
            value = conv.to_python(value)
        except ValidationError:
            return None
        if value not in dict(self.choices):
            return None
        return value

    def to_python(self, value):
        if value == '':
            #XXX: check for multiple?
            if self.multiple:
                return []
            return None
        if self.multiple:
            value = [item for item in map(self._safe_to_python, value or [])
                     if item is not None]
        else:
            value = self._safe_to_python(value)
        return value

    def __iter__(self):
        conv = self.conv(field=self.field)
        for python_value, label in self.choices:
            yield conv.from_python(python_value), label

    def get_label(self, value):
        conv = self.conv(field=self.field)
        return dict(self.choices).get(conv.to_python(value))


class BaseDatetime(Converter):

    format = None
    readable_format = None
    replacements = (('%H', 'HH'), ('%M', 'MM'), ('%d', 'DD'),
                    ('%m', 'MM'), ('%Y', 'YYYY'))
    error_wrong_format = N_('Wrong format (%(readable_format)s)')

    def __init__(self, *args, **kwargs):
        if not 'readable_format' in kwargs or 'format' in kwargs:
            replacements = self.replacements # XXX make language-dependent
            fmt = kwargs.get('format', self.format)
            for repl in replacements:
                fmt = fmt.replace(*repl)
            kwargs['readable_format'] = fmt
        Converter.__init__(self, *args, **kwargs)

    def from_python(self, value):
        if value is None:
            return ''
        # carefull to years before 1900
        return strftime(value, self.format)

    def to_python(self, value):
        if not value:
            return None
        try:
            return self.convert_datetime(value)
        except ValueError:
            raise ValidationError(self.error_wrong_format)
        except TypeError, e:
            raise ValidationError, unicode(e)


class Datetime(BaseDatetime):

    format = '%d.%m.%Y, %H:%M'

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format)


class Date(BaseDatetime):

    format = '%d.%m.%Y'

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format).date()


class Time(BaseDatetime):

    format = '%H:%M'

    def from_python(self, value):
        if value is None:
            return ''
        # we don't care about year in time converter, so use native strftime
        return value.strftime(self.format)

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format).time()


class SplitDateTime(Converter):

    def from_python(self, value):
        return {'date':value, 'time':value}

    def to_python(self, value):
        if value['date'] is None:
            return None
        res = datetime.combine(value['date'], value['time'])
        return res


class Joiner(object):

    def join(self, values):
        return values

    def split(self, value):
        return value

    def __call__(self):
        return self.__class__()


class DatetimeJoiner(Joiner):
    # XXX Two classes to split and join datetimes?
    def join(self, values):
        return datetime.combine(*values)

    def split(self, value):
        if not value:
            return None, None
        return value.date(), value.time()


class Html(Char):
    '''
    Converter for flexible cleanup of HTML document fragments.
    A subclass of :class:`Char<insanities.forms.convs.Char>`.

    Uses :class:`utils.html.Sanitizer<insanities.utils.html.Sanitizer>`
    instance to sanitize input HTML.

    Construtor collects from given kwargs all of
    :class:`Sanitizer<insanities.utils.html.Sanitizer>`
    options and passes them into Sanitizer's constructor.

    For list properties it is allowed to use :meth:`add_%s` interface::

        Html(add_allowed_elements=['span'], add_dom_callbacks=[myfunc])
    '''

    allowed_elements = frozenset(('a', 'p', 'br', 'li', 'ul', 'ol', 'hr', 'u',
                                  'i', 'b', 'blockquote', 'sub', 'sup'))
    allowed_attributes = frozenset(('href', 'src', 'alt', 'title', 'class', 'rel'))
    drop_empty_tags = frozenset(('p', 'a', 'u', 'i', 'b', 'sub', 'sup'))
    allowed_classes = {}
    #: Function returning object marked safe for template engine.
    #: For example: jinja Markup object
    Markup = lambda s, x: x

    def _load_arg(self, kwargs, opt):
        if hasattr(self, opt):
            kwargs.setdefault(opt, getattr(self, opt))

    def __init__(self, **kwargs):
        from ..utils.html import PROPERTIES, LIST_PROPERTIES

        for opt in PROPERTIES:
            if not opt in kwargs:
                # passed throught kwargs set is stronger then stored in class
                # add_%  
                self._load_arg(kwargs, 'add_' + opt)
            self._load_arg(kwargs, opt)

        for opt in LIST_PROPERTIES:
            add_key = 'add_' + opt
            if add_key in kwargs:
                kwargs[opt] = set(kwargs.get(opt, LIST_PROPERTIES[opt]))
                kwargs[opt].update(kwargs.pop(add_key))

        super(Html, self).__init__(**kwargs)

    def clean_value(self, value):
        from ..utils.html import ParseError, Sanitizer

        value = super(Html, self).clean_value(value)
        sanitizer = Sanitizer(**self._init_kwargs)
        try:
            clean = sanitizer.sanitize(value)
        except ParseError:
            raise ValidationError(u'not valid html')
        else:
            return self.Markup(clean)

    @property
    def tags(self):
        return self.allowed_elements
    @property
    def attrs(self):
        return self.allowed_attributes


class List(Converter):

    filter = None

    def from_python(self, value):
        result = OrderedDict()
        for index, item in enumerate(value):
            result[str(index+1)] = item
        return result

    def to_python(self, value):
        items = value.values()
        if self.filter is not None:
            items = filter(self.filter, items)
        return items

class SimpleFile(Converter):

    def _is_empty(self, file):
        return file == u'' or file is None #XXX WEBOB ONLY !!!

    def to_python(self, file):
        if not self._is_empty(file):
            return file

    def from_python(self, value):
        return None




# -*- coding: utf-8 -*-

import re
from . import convs, widgets, fields


class PasswordConv(convs.Char):

    def from_python(self, value):
        return dict([(field.name, None) for field in self.field.fields])

    def get_default(self):
        return ''

    def to_python(self, value):
        etalon = value[list(value)[0]]
        for field in self.field.fields:
            self._assert(value[field.name] == etalon,
                         'password and confirm mismatch', 'mismatch')
        self._assert(etalon not in (None, '')  or self.null,
                     'password required', 'required')
        return etalon


def PasswordSet(name='password',
                 min_length=3, max_length=200, null=True,
                 password_label=None, confirm_label='confirmmm',
                 **kwargs):
        # class implementation has problem with Fieldset copying:
        # it requires to save all kwargs in object's __dict__
        char = convs.Char(min_length=min_length, max_length=max_length, null=null)
        items = (('pass', password_label), ('conf', confirm_label))
        
        kwargs['fields'] = [fields.Field(subfieldname,
                                         conv=char,
                                         label=label,
                                         widget=widgets.PasswordInput)
                            for subfieldname, label in items]
        kwargs.setdefault('conv', PasswordConv(null=null))
        kwargs.setdefault('template', 'fieldset-line')
        
        return fields.FieldSet(name, get_default=lambda: '', **kwargs)




# -*- coding: utf-8 -*-

from xml.sax import saxutils
import weakref, re, sys


def quoteattr(value):
    """
    works like quoteattr from saxutils
    """
    if value == '':
        return '""'
    return '%s' % saxutils.escape(unicode(value), {'"': '&quot;'})

def quoteattrs(data):
    """
    takes dict of attrs and return correct
    xhtml representation
    """
    items = []
    for key, value in data.items():
        items.append('%s="%s"' % (key, quoteattr(value)))
    return ' '.join(items)

def quote_js(text):
    '''Quotes text to be used as JavaScript string in HTML templates. The
    result doesn't contain surrounding quotes.'''
    text = text.replace('\\', '\\\\');
    text = text.replace('\n', '\\n');
    text = text.replace('\r', '');
    for char in '\'"<>&':
        text = text.replace(char, '\\x%2x' % ord(char))
    return text

def weakproxy(obj):
    '''Safe version of weakref.proxy.'''
    try:
        obj = weakref.proxy(obj)
    except TypeError:
        pass
    return obj


class cached_property(object):
    '''Turns decorated method into caching property (method is called once on
    first access to property).'''

    def __init__(self, method, name=None):
        self.method = method
        self.name = name or method.__name__
        self.__doc__ = method.__doc__

    def __get__(self, inst, cls):
        if inst is None:
            return self
        result = self.method(inst)
        setattr(inst, self.name, result)
        return result


# http://www.w3.org/TR/REC-xml/#NT-Char
# Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | 
#          [#x10000- #x10FFFF]
# (any Unicode character, excluding the surrogate blocks, FFFE, and FFFF)
_char_tail = ''
if sys.maxunicode > 0x10000:
    _char_tail = u'%s-%s' % (unichr(0x10000),
                             unichr(min(sys.maxunicode, 0x10FFFF)))
_nontext_sub = re.compile(
                ur'[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD%s]' % _char_tail,
                re.U).sub
def replace_nontext(text, replacement=u'\uFFFD'):
    return _nontext_sub(replacement, text)

def conf_to_dict(cfg):
    '''Selects all module members which names are upper case and return dict'''
    keys = filter(lambda x: x.isupper(), dir(cfg))
    d = {}
    for key in keys:
        d[key] = getattr(cfg, key)
    return d


# i18n markers
def N_(msg):
    '''Single translatable string marker'''
    return msg

class M_(unicode):
    '''Marker for translatable string with plural form'''
    def __new__(cls, single, plural):
        self = unicode.__new__(cls, single)
        self.plural = plural
        return self

def smart_gettext(translation, msg, count=None):
    '''If msg is instance of M_ returns multiple translation
    otherwise return single translation'''
    if count is None:
        count = 1
    if isinstance(msg, M_) and count is not None:
        return translation.ungettext(msg, msg.plural, count)
    return translation.ugettext(msg)


